@@ -96,6 +96,7 @@ impl CallBuilder {
     }
 
     /// Invokes the call with the specified callbacks
+    #[cfg(target_arch = "wasm32")]
     pub fn invoke(mut self) -> Call {
         let call = self.call.take().unwrap();
         match &self.request {
@@ -115,6 +116,11 @@ impl CallBuilder {
 
         call
     }
+
+    #[cfg(not(target_arch = "wasm32"))]
+    pub fn invoke(self) -> Call {
+        panic!("invoke not supported on this platform");
+    }
 }
 
 impl Drop for CallBuilder {
@@ -139,7 +145,9 @@ impl Call {
 
     /// Upon receiving a particular message from the service that is
     /// invoked this callback will take some action
+    ///
     /// Note: This must be called before the invoke or things will go wrong
+    /// hence there is a builder that invokes this in the right order
     fn callback<C, F>(&mut self, mut callback: F) -> &mut Self
     where
         C: Serialize + de::DeserializeOwned + Send + Sync + 'static,
@@ -150,7 +158,7 @@ impl Call {
             callback(req);
             Ok(())
         };
-        let recv = super::recv_internal(Some(self.handle), callback);
+        let recv = super::callback_internal(self.handle, callback);
         self.callbacks.lock().unwrap().push(recv);
         self
     }
@@ -186,8 +194,9 @@ where
 
     /// Waits for the call to complete and returns the response from
     /// the server
+    #[cfg(feature = "rt")]
     pub fn wait(self) -> Result<T, CallError> {
-        crate::backend::block_on::block_on(self)
+        crate::task::block_on(self)
     }
 
     /// Tries to get the result of the call to the server but will not
