@@ -4,7 +4,14 @@ mod error;
 mod handle;
 mod recv;
 mod reply;
+#[cfg(feature = "syscalls")]
 pub(crate) mod syscall;
+#[cfg(not(feature = "syscalls"))]
+pub(crate) mod unsupported;
+
+#[allow(unused_imports)]
+#[cfg(not(feature = "syscalls"))]
+pub(crate) use unsupported as syscall;
 
 use serde::*;
 use std::any::type_name;
@@ -43,17 +50,18 @@ where
     CallBuilder::new(call, req)
 }
 
-pub fn recv<RES, REQ, F>(callback: F) -> Recv
+pub fn listen<RES, REQ, F>(_callback: F) -> Recv
 where
     REQ: de::DeserializeOwned + Send + Sync + 'static,
     RES: Serialize + Send + Sync + 'static,
     F: Fn(REQ) -> Result<RES, CallError>,
     F: Send + Sync + 'static,
 {
-    recv_internal::<RES, REQ, F>(None, callback)
+    panic!("listen not supported on this platform");
 }
 
-pub(crate) fn recv_internal<RES, REQ, F>(parent: Option<CallHandle>, callback: F) -> Recv
+#[cfg(target_arch = "wasm32")]
+pub(crate) fn callback_internal<RES, REQ, F>(parent: CallHandle, callback: F) -> Recv
 where
     REQ: de::DeserializeOwned + Send + Sync + 'static,
     RES: Serialize + Send + Sync + 'static,
@@ -61,14 +69,25 @@ where
     F: Send + 'static,
 {
     let topic = type_name::<REQ>();
-    let recv = crate::engine::BusEngine::recv(callback);
+    let recv = crate::engine::BusEngine::callback(callback);
     let handle = recv.handle;
 
-    syscall::recv(parent, handle, topic);
-
-    recv
+    syscall::callback(parent, handle, topic);
+    return recv;
 }
 
+#[cfg(not(target_arch = "wasm32"))]
+pub(crate) fn callback_internal<RES, REQ, F>(_parent: CallHandle, _callback: F) -> Recv
+where
+    REQ: de::DeserializeOwned + Send + Sync + 'static,
+    RES: Serialize + Send + Sync + 'static,
+    F: FnMut(REQ) -> Result<RES, CallError>,
+    F: Send + 'static,
+{
+    panic!("recv not supported on this platform");
+}
+
+#[cfg(target_arch = "wasm32")]
 pub(self) fn reply<RES>(handle: CallHandle, response: RES)
 where
     RES: Serialize,
@@ -78,13 +97,33 @@ where
             syscall::reply(handle, &res[..]);
         }
         Err(_err) => syscall::error(handle, CallError::SerializationFailed as i32),
-    }
+    };
 }
 
+#[cfg(not(target_arch = "wasm32"))]
+pub(self) fn reply<RES>(_handle: CallHandle, _response: RES)
+where
+    RES: Serialize,
+{
+    panic!("reply not supported on this platform");
+}
+
+#[cfg(target_arch = "wasm32")]
 pub(self) fn drop(handle: CallHandle) {
     syscall::drop(handle);
 }
 
+#[cfg(not(target_arch = "wasm32"))]
+pub(self) fn drop(_handle: CallHandle) {
+    panic!("drop handle not supported on this platform");
+}
+
+#[cfg(target_arch = "wasm32")]
 pub fn thread_id() -> u32 {
     syscall::thread_id()
 }
+
+#[cfg(not(target_arch = "wasm32"))]
+pub fn thread_id() -> u32 {
+    panic!("thread_id not supported on this platform");
+}
