@@ -1,3 +1,4 @@
+#![allow(dead_code)]
 use super::*;
 #[allow(unused_imports, dead_code)]
 use tracing::{debug, error, info, trace, warn};
@@ -5,6 +6,8 @@ use tracing::{debug, error, info, trace, warn};
 mod raw {
     use super::*;
 
+    // Frees memory that was passed to the operating system by
+    // the program
     #[no_mangle]
     pub extern "C" fn wasm_bus_free(buf_ptr: u32, buf_len: u32) {
         trace!("wasm_bus_free (buf={} bytes)", buf_len);
@@ -14,6 +17,8 @@ mod raw {
         }
     }
 
+    // Allocates memory that will be used to pass data from the
+    // operating system back to this program
     #[no_mangle]
     pub extern "C" fn wasm_bus_malloc(len: u32) -> u32 {
         trace!("wasm_bus_malloc (len={})", len);
@@ -23,11 +28,45 @@ mod raw {
         return ptr as u32;
     }
 
-    // Invoked when the call has finished
+    // Invoked by the operating system when during a poll when a
+    // request is to be processed by this program
     #[no_mangle]
-    pub extern "C" fn wasm_bus_data(handle: u32, data: u32, data_len: u32) {
+    pub extern "C" fn wasm_bus_start(
+        handle: u32,
+        topic_ptr: u32,
+        topic_len: u32,
+        request_ptr: u32,
+        request_len: u32,
+    ) {
+        let topic = unsafe {
+            let topic =
+                Vec::from_raw_parts(topic_ptr as *mut u8, topic_len as usize, topic_len as usize);
+            String::from_utf8_lossy(&topic[..]).to_string()
+        };
         trace!(
-            "wasm_bus_data (handle={}, response={} bytes)",
+            "wasm_bus_start (handle={}, topic={}, request={} bytes)",
+            handle,
+            topic,
+            request_len
+        );
+        unsafe {
+            let _request = Vec::from_raw_parts(
+                request_ptr as *mut u8,
+                request_len as usize,
+                request_len as usize,
+            );
+        }
+
+        #[cfg(feature = "rt")]
+        crate::task::wake();
+    }
+
+    // Invoked by the operating system when a call has finished
+    // This call includes the data that was returned
+    #[no_mangle]
+    pub extern "C" fn wasm_bus_finish(handle: u32, data: u32, data_len: u32) {
+        trace!(
+            "wasm_bus_finish (handle={}, response={} bytes)",
             handle,
             data_len
         );
@@ -35,24 +74,14 @@ mod raw {
             let response =
                 Vec::from_raw_parts(data as *mut u8, data_len as usize, data_len as usize);
 
-            match crate::engine::BusEngine::put(handle.into(), response) {
-                Some(Ok(response)) => {
-                    let _response_len = response.len();
-                    let _response = response.as_ptr();
-                    //reply(handle, response as i32, response_len as i32);
-                }
-                Some(Err(_err)) => {
-                    //fault(handle, err as i32);
-                }
-                None => {}
-            };
+            let _ = crate::engine::BusEngine::put(handle.into(), response);
         }
 
         #[cfg(feature = "rt")]
         crate::task::wake();
     }
 
-    // Invoked when the call has failed
+    // Invoked by the operating system when the call this program made has failed
     #[no_mangle]
     pub extern "C" fn wasm_bus_error(handle: u32, error: u32) {
         trace!("wasm_bus_err (handle={}, error={})", handle, error);
@@ -64,13 +93,24 @@ mod raw {
 
     #[link(wasm_import_module = "wasm-bus")]
     extern "C" {
-        pub(crate) fn drop(handle: u32);
+        // Returns a handle 32-bit number which is used while generating
+        // handles for calls and receive hooks
         pub(crate) fn rand() -> u32;
-        pub(crate) fn yield_and_wait(timeout_ms: u32);
 
+        // Indicates that a fault has occured while processing a call
         pub(crate) fn fault(handle: u32, error: i32);
+
+        // Returns the response of a listen invokation to a program
+        // from the operating system
         pub(crate) fn reply(handle: u32, response: i32, response_len: i32);
 
+        // Drops a handle used by calls or callbacks
+        pub(crate) fn drop(handle: u32);
+
+        // Calls a function using the operating system call to find
+        // the right target based on the wapm and topic.
+        // The operating system will respond with either a 'wasm_bus_finish'
+        // or a 'wasm_bus_error' message.
         pub(crate) fn call(
             parent: u32,
             handle: u32,
@@ -81,8 +121,20 @@ mod raw {
             request: i32,
             request_len: i32,
         ) -> u32;
-        pub(crate) fn recv(parent: u32, handle: u32, topic: i32, topic_len: i32);
 
+        // Incidates that a call that will be made should invoke a callback
+        // back to this process under the designated handle.
+        pub(crate) fn callback(parent: u32, handle: u32, topic: i32, topic_len: i32);
+
+        // Tells the operating system that this program is ready to respond
+        // to calls on a particular topic name.
+        pub(crate) fn listen(topic: i32, topic_len: i32);
+
+        // Polls the operating system for messages which will be returned via
+        // the 'wasm_bus_start' function call.
+        pub(crate) fn poll();
+
+        // Returns a unqiue ID for the thread
         pub(crate) fn thread_id() -> u32;
     }
 }
@@ -90,6 +142,7 @@ mod raw {
 pub fn drop(handle: CallHandle) {
     unsafe { raw::drop(handle.id) }
 }
+
 pub fn rand() -> u32 {
     unsafe { raw::rand() }
 }
@@ -100,6 +153,18 @@ pub fn error(handle: CallHandle, error: i32) {
     }
 }
 
+pub fn poll() {
+    unsafe { raw::poll() }
+}
+
+pub fn listen(topic: &str) {
+    unsafe {
+        let topic_len = topic.len();
+        let topic = topic.as_ptr();
+        raw::listen(topic as i32, topic_len as i32)
+    }
+}
+
 pub fn reply(handle: CallHandle, response: &[u8]) {
     unsafe {
         let response_len = response.len();
@@ -140,18 +205,11 @@ pub fn call(
     }
 }
 
-pub fn recv(parent: Option<CallHandle>, handle: CallHandle, topic: &str) {
+pub fn callback(parent: CallHandle, handle: CallHandle, topic: &str) {
     unsafe {
-        let parent = parent.map(|a| a.id).unwrap_or_else(|| u32::MAX);
         let topic_len = topic.len();
         let topic = topic.as_ptr();
-        raw::recv(parent, handle.id, topic as i32, topic_len as i32)
-    }
-}
-
-pub fn yield_and_wait(timeout_ms: u32) {
-    unsafe {
-        raw::yield_and_wait(timeout_ms);
+        raw::callback(parent.id, handle.id, topic as i32, topic_len as i32)
     }
 }
 
