@@ -1,6 +1,5 @@
-pub mod block_on;
+pub mod fuse;
 pub mod process;
 pub mod reqwest;
 pub mod time;
 pub mod ws;
-pub mod yield_waker;
